@@ -12,12 +12,6 @@ from .util import HMaccas
 if TYPE_CHECKING:
     from .svchub import SvcHub
 
-try:
-    raise Exception()
-except:
-    pass
-
-
 class BrokerThr(BrokerCli):
     """external api; behaves like BrokerMP but using plain threads"""
 
@@ -63,6 +57,10 @@ class BrokerThr(BrokerCli):
             self.httpsrv.listen(args[0], 1)
             return
 
+        if dest == "set_netdevs":
+            self.httpsrv.set_netdevs(args[0])
+            return
+
         # new ipc invoking managed service in hub
         obj = self.hub
         for node in dest.split("."):
