@@ -25,15 +25,13 @@ from .th_srv import HAVE_PIL, HAVE_VIPS
 from .u2idx import U2idx
 from .util import HMaccas, shut_socket
 
-try:
-    raise Exception()
-except:
-    pass
-
 if TYPE_CHECKING:
     from .httpsrv import HttpSrv
 
 
+PTN_HTTP = re.compile(br"[A-Z]{3}[A-Z ]")
+
+
 class HttpConn(object):
     """
     spawned by HttpSrv to handle an incoming client connection,
@@ -45,6 +43,7 @@ class HttpConn(object):
     )  :
         self.s = sck
         self.sr  = None
+        self.cli  = None
         self.addr = addr
         self.hsrv = hsrv
 
@@ -55,6 +54,8 @@ class HttpConn(object):
         self.cert_path = hsrv.cert_path
         self.u2fh  = hsrv.u2fh  # mypy404
         self.iphash  = hsrv.broker.iphash
+        self.bans   = hsrv.bans
+        self.aclose   = hsrv.aclose
 
         enth = (HAVE_PIL or HAVE_VIPS or HAVE_FFMPEG) and not self.args.no_thumb
         self.thumbcli  = ThumbCli(hsrv) if enth else None  # mypy404
@@ -62,7 +63,7 @@ class HttpConn(object):
 
         self.t0  = time.time()  # mypy404
         self.stopping = False
-        self.nreq  = 0  # mypy404
+        self.nreq  = -1  # mypy404
         self.nbyte  = 0  # mypy404
         self.u2idx  = None
         self.log_func  = hsrv.log  # mypy404
@@ -134,9 +135,11 @@ class HttpConn(object):
                 self.s.send(b"HTTP/1.1 400 Bad Request\r\n\r\n" + err.encode("utf-8"))
                 return False
 
-        return method not in [None, b"GET ", b"HEAD", b"POST", b"PUT ", b"OPTI"]
+        return not method or not bool(PTN_HTTP.match(method))
 
     def run(self)  :
+        self.s.settimeout(10)
+
         self.sr = None
         if self.args.https_only:
             is_https = True
@@ -205,6 +208,6 @@ class HttpConn(object):
 
         while not self.stopping:
             self.nreq += 1
-            cli = HttpCli(self)
-            if not cli.run():
+            self.cli = HttpCli(self)
+            if not self.cli.run():
                 return
