@@ -3,20 +3,13 @@ from __future__ import print_function, unicode_literals
 
 import calendar
 import time
+import stat
 import zlib
 
 from .bos import bos
 from .sutil import StreamArc, errdesc
 from .util import min_ex, sanitize_fn, spack, sunpack, yieldfile
 
-try:
-    raise Exception()
-
-    from .util import NamedLogger
-except:
-    pass
-
-
 def dostime2unix(buf )  :
     t, d = sunpack(b"<HH", buf)
 
@@ -240,6 +233,9 @@ class StreamZip(StreamArc):
         src = f["ap"]
         st = f["st"]
 
+        if stat.S_ISDIR(st.st_mode):
+            return
+
         sz = st.st_size
         ts = st.st_mtime
 
@@ -271,6 +267,7 @@ class StreamZip(StreamArc):
             yield self._ct(buf)
 
     def gen(self)    :
+        errf   = {}
         errors = []
         try:
             for f in self.fgen:
@@ -311,5 +308,5 @@ class StreamZip(StreamArc):
             ecdr, _ = gen_ecdr(self.items, cdir_pos, cdir_end)
             yield self._ct(ecdr)
         finally:
-            if errors:
+            if errf:
                 bos.unlink(errf["ap"])
