@@ -9,11 +9,6 @@ from .bos import bos
 from .th_srv import HAVE_WEBP, thumb_path
 from .util import Cooldown
 
-try:
-    raise Exception()
-except:
-    pass
-
 if TYPE_CHECKING:
     from .httpsrv import HttpSrv
 
@@ -30,6 +25,8 @@ class ThumbCli(object):
 
         try:
             c = hsrv.th_cfg
+            if not c:
+                raise Exception()
         except:
             c = {k: {} for k in ["thumbable", "pil", "vips", "ffi", "ffv", "ffa"]}
 
