@@ -3,8 +3,10 @@ from __future__ import print_function, unicode_literals
 
 import base64
 import contextlib
+import errno
 import hashlib
 import hmac
+import logging
 import math
 import mimetypes
 import os
@@ -22,13 +24,37 @@ import time
 import traceback
 from collections import Counter
 from datetime import datetime
+from email.utils import formatdate
 
+from ipaddress import IPv4Address, IPv4Network, IPv6Address, IPv6Network
 from queue import Queue
 
 from .__init__ import ANYWIN, MACOS, PY2, TYPE_CHECKING, VT100, WINDOWS
 from .__version__ import S_BUILD_DT, S_VERSION
 from .stolen import surrogateescape
 
+
+def _ens(want )   :
+    ret  = []
+    for v in want.split():
+        try:
+            ret.append(getattr(errno, v))
+        except:
+            pass
+
+    return tuple(ret)
+
+
+# WSAECONNRESET - foribly closed by remote
+# WSAENOTSOCK - no longer a socket
+# EUNATCH - can't assign requested address (wifi down)
+E_SCK = _ens("ENOTCONN EUNATCH EBADF WSAENOTSOCK WSAECONNRESET")
+E_ADDR_NOT_AVAIL = _ens("EADDRNOTAVAIL WSAEADDRNOTAVAIL")
+E_ADDR_IN_USE = _ens("EADDRINUSE WSAEADDRINUSE")
+E_ACCESS = _ens("EACCES WSAEACCES")
+E_UNREACH = _ens("EHOSTUNREACH WSAEHOSTUNREACH ENETUNREACH WSAENETUNREACH")
+
+
 try:
     import ctypes
     import fcntl
@@ -36,11 +62,6 @@ try:
 except:
     pass
 
-try:
-    from ipaddress import IPv6Address
-except:
-    pass
-
 try:
     HAVE_SQLITE3 = True
     import sqlite3  # pylint: disable=unused-import  # typechk
@@ -53,24 +74,6 @@ try:
 except:
     HAVE_PSUTIL = False
 
-try:
-    raise Exception()
-    raise Exception()
-
-    raise Exception()
-    raise Exception()
-
-    class RootLogger(Protocol):
-        def __call__(self, src , msg , c   = 0)  :
-            return None
-
-    class NamedLogger(Protocol):
-        def __call__(self, msg , c   = 0)  :
-            return None
-
-except:
-    pass
-
 if TYPE_CHECKING:
     import magic
 
@@ -79,10 +82,9 @@ if TYPE_CHECKING:
 FAKE_MP = False
 
 try:
-    if not FAKE_MP:
-        import multiprocessing as mp
-    else:
-        import multiprocessing.dummy as mp  # type: ignore
+    import multiprocessing as mp
+
+    # import multiprocessing.dummy as mp
 except ImportError:
     # support jython
     mp = None  # type: ignore
@@ -120,26 +122,31 @@ else:
     FS_ENCODING = sys.getfilesystemencoding()
 
 
-SYMTIME = sys.version_info >= (3, 6) and os.utime in os.supports_follow_symlinks
-
-HTTP_TS_FMT = "%a, %d %b %Y %H:%M:%S GMT"
+SYMTIME = sys.version_info > (3, 6) and os.utime in os.supports_follow_symlinks
 
 META_NOBOTS = '<meta name="robots" content="noindex, nofollow">'
 
 HTTPCODE = {
     200: "OK",
+    201: "Created",
     204: "No Content",
     206: "Partial Content",
+    207: "Multi-Status",
+    301: "Moved Permanently",
     302: "Found",
     304: "Not Modified",
     400: "Bad Request",
+    401: "Unauthorized",
     403: "Forbidden",
     404: "Not Found",
     405: "Method Not Allowed",
+    409: "Conflict",
     411: "Length Required",
+    412: "Precondition Failed",
     413: "Payload Too Large",
     416: "Requested Range Not Satisfiable",
     422: "Unprocessable Entity",
+    423: "Locked",
     429: "Too Many Requests",
     500: "Internal Server Error",
     501: "Not Implemented",
@@ -156,7 +163,27 @@ IMPLICATIONS = [
     ["e2vu", "e2v"],
     ["e2vp", "e2v"],
     ["e2v", "e2d"],
+    ["smbw", "smb"],
+    ["smb1", "smb"],
+    ["smbvvv", "smbvv"],
+    ["smbvv", "smbv"],
+    ["smbv", "smb"],
+    ["zv", "zmv"],
+    ["zv", "zsv"],
+    ["z", "zm"],
+    ["z", "zs"],
+    ["zmvv", "zmv"],
+    ["zm4", "zm"],
+    ["zm6", "zm"],
+    ["zmv", "zm"],
+    ["zms", "zm"],
+    ["zsv", "zs"],
 ]
+if ANYWIN:
+    IMPLICATIONS.extend([["z", "zm4"]])
+
+
+UNPLICATIONS = [["no_dav", "daw"]]
 
 
 MIMES = {
@@ -258,7 +285,7 @@ def py_desc()  :
         bitness = struct.calcsize("P") * 8
 
     host_os = platform.system()
-    compiler = platform.python_compiler()
+    compiler = platform.python_compiler().split("http")[0]
 
     m = re.search(r"([0-9]+\.[0-9\.]+)", platform.version())
     os_ver = m.group(1) if m else ""
@@ -311,6 +338,37 @@ _  = (mp, BytesIO, quote, unquote, SQLITE_VER, JINJA_VER, PYFTPD_VER)
 __all__ = ["mp", "BytesIO", "quote", "unquote", "SQLITE_VER", "JINJA_VER", "PYFTPD_VER"]
 
 
+class Daemon(threading.Thread):
+    def __init__(
+        self,
+        target ,
+        name  = None,
+        a  = None,
+        r  = True,
+    )  :
+        threading.Thread.__init__(self, target=target, name=name, args=a or ())
+        self.daemon = True
+        if r:
+            self.start()
+
+
+class Netdev(object):
+    def __init__(self, ip , idx , name , desc ):
+        self.ip = ip
+        self.idx = idx
+        self.name = name
+        self.desc = desc
+
+    def __str__(self):
+        return "{}-{}{}".format(self.idx, self.name, self.desc)
+
+    def __lt__(self, rhs):
+        return str(self) < str(rhs)
+
+    def __eq__(self, rhs):
+        return str(self) == str(rhs)
+
+
 class Cooldown(object):
     def __init__(self, maxage )  :
         self.maxage = maxage
@@ -337,6 +395,94 @@ class Cooldown(object):
             return ret
 
 
+class HLog(logging.Handler):
+    def __init__(self, log_func )  :
+        logging.Handler.__init__(self)
+        self.log_func = log_func
+        self.ptn_ftp = re.compile(r"^([0-9a-f:\.]+:[0-9]{1,5})-\[")
+        self.ptn_smb_ign = re.compile(r"^(Callback added|Config file parsed)")
+
+    def __repr__(self)  :
+        level = logging.getLevelName(self.level)
+        return "<%s cpp(%s)>" % (self.__class__.__name__, level)
+
+    def flush(self)  :
+        pass
+
+    def emit(self, record )  :
+        msg = self.format(record)
+        lv = record.levelno
+        if lv < logging.INFO:
+            c = 6
+        elif lv < logging.WARNING:
+            c = 0
+        elif lv < logging.ERROR:
+            c = 3
+        else:
+            c = 1
+
+        if record.name == "pyftpdlib":
+            m = self.ptn_ftp.match(msg)
+            if m:
+                ip = m.group(1)
+                msg = msg[len(ip) + 1 :]
+                if ip.startswith("::ffff:"):
+                    record.name = ip[7:]
+                else:
+                    record.name = ip
+        elif record.name.startswith("impacket"):
+            if self.ptn_smb_ign.match(msg):
+                return
+
+        self.log_func(record.name[-21:], msg, c)
+
+
+class NetMap(object):
+    def __init__(self, ips , netdevs  )  :
+        if "::" in ips:
+            ips = [x for x in ips if x != "::"] + list(
+                [x.split("/")[0] for x in netdevs if ":" in x]
+            )
+            ips.append("0.0.0.0")
+
+        if "0.0.0.0" in ips:
+            ips = [x for x in ips if x != "0.0.0.0"] + list(
+                [x.split("/")[0] for x in netdevs if ":" not in x]
+            )
+
+        ips = [x for x in ips if x not in ("::1", "127.0.0.1")]
+        ips = [[x for x in netdevs if x.startswith(y + "/")][0] for y in ips]
+
+        self.cache   = {}
+        self.b2sip   = {}
+        self.b2net    = {}
+        self.bip  = []
+        for ip in ips:
+            v6 = ":" in ip
+            fam = socket.AF_INET6 if v6 else socket.AF_INET
+            bip = socket.inet_pton(fam, ip.split("/")[0])
+            self.bip.append(bip)
+            self.b2sip[bip] = ip.split("/")[0]
+            self.b2net[bip] = (IPv6Network if v6 else IPv4Network)(ip, False)
+
+        self.bip.sort(reverse=True)
+
+    def map(self, ip )  :
+        try:
+            return self.cache[ip]
+        except:
+            pass
+
+        v6 = ":" in ip
+        ci = IPv6Address(ip) if v6 else IPv4Address(ip)
+        bip = next((x for x in self.bip if ci in self.b2net[x]), None)
+        ret = self.b2sip[bip] if bip else ""
+        if len(self.cache) > 9000:
+            self.cache = {}
+        self.cache[ip] = ret
+        return ret
+
+
 class UnrecvEOF(OSError):
     pass
 
@@ -357,7 +503,16 @@ class _Unrecv(object):
             self.buf = self.buf[nbytes:]
             return ret
 
-        ret = self.s.recv(nbytes)
+        while True:
+            try:
+                ret = self.s.recv(nbytes)
+                break
+            except socket.timeout:
+                continue
+            except:
+                ret = b""
+                break
+
         if not ret:
             raise UnrecvEOF("client stopped sending data")
 
@@ -444,6 +599,27 @@ class _LUnrecv(object):
 Unrecv = _Unrecv
 
 
+class CachedSet(object):
+    def __init__(self, maxage )  :
+        self.c   = {}
+        self.maxage = maxage
+        self.oldest = 0.0
+
+    def add(self, v )  :
+        self.c[v] = time.time()
+
+    def cln(self)  :
+        now = time.time()
+        if now - self.oldest < self.maxage:
+            return
+
+        c = self.c = {k: v for k, v in self.c.items() if now - v < self.maxage}
+        try:
+            self.oldest = c[min(c, key=c.get)]
+        except:
+            self.oldest = now
+
+
 class FHC(object):
     class CE(object):
         def __init__(self, fh )  :
@@ -540,9 +716,7 @@ class MTHash(object):
         self.done_q     = Queue()
         self.thrs = []
         for n in range(cores):
-            t = threading.Thread(target=self.worker, name="mth-" + str(n))
-            t.daemon = True
-            t.start()
+            t = Daemon(self.worker, "mth-" + str(n))
             self.thrs.append(t)
 
     def hash(
@@ -643,6 +817,9 @@ class HMaccas(object):
         try:
             return self.cache[msg]
         except:
+            if len(self.cache) > 9000:
+                self.cache = {}
+
             zb = hmac.new(self.key, msg, hashlib.sha512).digest()
             zs = base64.urlsafe_b64encode(zb)[: self.retlen].decode("utf-8")
             self.cache[msg] = zs
@@ -743,7 +920,7 @@ class Garda(object):
         if not self.lim:
             return 0, ip
 
-        if ":" in ip and not PY2:
+        if ":" in ip:
             # assume /64 clients; drop 4 groups
             ip = IPv6Address(ip).exploded[:-20]
 
@@ -845,20 +1022,14 @@ def alltrace()  :
         else:
             rret += ret
 
-    return "\n".join(rret + bret)
+    return "\n".join(rret + bret) + "\n"
 
 
 def start_stackmon(arg_str , nid )  :
     suffix = "-{}".format(nid) if nid else ""
     fp, f = arg_str.rsplit(",", 1)
     zi = int(f)
-    t = threading.Thread(
-        target=stackmon,
-        args=(fp, zi, suffix),
-        name="stackmon" + suffix,
-    )
-    t.daemon = True
-    t.start()
+    Daemon(stackmon, "stackmon" + suffix, (fp, zi, suffix))
 
 
 def stackmon(fp , ival , suffix )  :
@@ -911,13 +1082,7 @@ def start_log_thrs(
         tname = "logthr-n{}-i{:x}".format(nid, os.getpid())
         lname = tname[3:]
 
-    t = threading.Thread(
-        target=log_thrs,
-        args=(logger, ival, lname),
-        name=tname,
-    )
-    t.daemon = True
-    t.start()
+    Daemon(log_thrs, tname, (logger, ival, lname))
 
 
 def log_thrs(log    , ival , name )  :
@@ -964,12 +1129,20 @@ def ren_open(
     fun = kwargs.pop("fun", open)
     fdir = kwargs.pop("fdir", None)
     suffix = kwargs.pop("suffix", None)
+    overwrite = kwargs.pop("overwrite", None)
 
     if fname == os.devnull:
         with fun(fname, *args, **kwargs) as f:
             yield {"orz": (f, fname)}
             return
 
+    if overwrite:
+        assert fdir
+        fpath = os.path.join(fdir, fname)
+        with fun(fsenc(fpath), *args, **kwargs) as f:
+            yield {"orz": (f, fname)}
+            return
+
     if suffix:
         ext = fname.split(".")[-1]
         if len(ext) < 7:
@@ -1003,6 +1176,7 @@ def ren_open(
 
             with fun(fsenc(fpath), *args, **kwargs) as f:
                 if b64:
+                    assert fdir
                     fp2 = "fn-trunc.{}.txt".format(b64)
                     fp2 = os.path.join(fdir, fp2)
                     with open(fsenc(fp2), "wb") as f2:
@@ -1014,7 +1188,7 @@ def ren_open(
         except OSError as ex_:
             ex = ex_
 
-            if ex.errno == 22 and not asciified:
+            if ex.errno == errno.EINVAL and not asciified:
                 asciified = True
                 bname, fname = [
                     zs.encode("ascii", "replace").decode("ascii").replace("?", "_")
@@ -1022,11 +1196,14 @@ def ren_open(
                 ]
                 continue
 
-            if ex.errno not in [36, 63] and (not WINDOWS or ex.errno != 22):
+            # ENOTSUP: zfs on ubuntu 20.04
+            if ex.errno not in (errno.ENAMETOOLONG, errno.ENOSR, errno.ENOTSUP) and (
+                not WINDOWS or ex.errno != errno.EINVAL
+            ):
                 raise
 
         if not b64:
-            zs = (orig_name + "\n" + suffix).encode("utf-8", "replace")
+            zs = "{}\n{}".format(orig_name, suffix).encode("utf-8", "replace")
             zs = hashlib.sha512(zs).digest()[:12]
             b64 = base64.urlsafe_b64encode(zs).decode("utf-8")
 
@@ -1115,7 +1292,7 @@ class MultipartParser(object):
                 return field, None
 
             try:
-                is_webkit = self.headers["user-agent"].lower().find("applewebkit") >= 0
+                is_webkit = "applewebkit" in self.headers["user-agent"].lower()
             except:
                 is_webkit = False
 
@@ -1176,6 +1353,7 @@ class MultipartParser(object):
                     buf = buf[d:]
 
                 # look for boundary near the end of the buffer
+                n = 0
                 for n in range(1, len(buf) + 1):
                     if not buf[-n:] in self.boundary:
                         n -= 1
@@ -1280,8 +1458,12 @@ def get_boundary(headers  )  :
 
 
 def read_header(sr )  :
+    t0 = time.time()
     ret = b""
     while True:
+        if time.time() - t0 > 120:
+            return []
+
         try:
             ret += sr.recv(1024)
         except:
@@ -1327,8 +1509,24 @@ def gen_filekey_dbg(
 
     assert log_ptn
     if log_ptn.search(fspath):
-        t = "fk({}) salt({}) size({}) inode({}) fspath({})"
-        log(t.format(ret[:8], salt, fsize, inode, fspath))
+        try:
+            import inspect
+
+            ctx = ",".join(inspect.stack()[n].function for n in range(2, 5))
+        except:
+            ctx = ""
+
+        p2 = "a"
+        try:
+            p2 = absreal(fspath)
+            if p2 != fspath:
+                raise Exception()
+        except:
+            t = "maybe wrong abspath for filekey;\norig: {}\nreal: {}"
+            log(t.format(fspath, p2), 1)
+
+        t = "fk({}) salt({}) size({}) inode({}) fspath({}) at({})"
+        log(t.format(ret[:8], salt, fsize, inode, fspath, ctx), 5)
 
     return ret
 
@@ -1336,8 +1534,7 @@ def gen_filekey_dbg(
 def gencookie(k , v , dur )  :
     v = v.replace(";", "")
     if dur:
-        dt = datetime.utcfromtimestamp(time.time() + dur)
-        exp = dt.strftime("%a, %d %b %Y %H:%M:%S GMT")
+        exp = formatdate(time.time() + dur, usegmt=True)
     else:
         exp = "Fri, 15 Aug 1997 01:00:00 GMT"
 
@@ -1489,9 +1686,12 @@ def exclude_dotfiles(filepaths )  :
     return [x for x in filepaths if not x.split("/")[-1].startswith(".")]
 
 
-def http_ts(ts )  :
-    file_dt = datetime.utcfromtimestamp(ts)
-    return file_dt.strftime(HTTP_TS_FMT)
+def ipnorm(ip )  :
+    if ":" in ip:
+        # assume /64 clients; drop 4 groups
+        return IPv6Address(ip).exploded[:-20]
+
+    return ip
 
 
 def html_escape(s , quot  = False, crlf  = False)  :
@@ -1516,17 +1716,21 @@ def html_bescape(s , quot  = False, crlf  = False)  :
     return s
 
 
-def quotep(txt )  :
+def _quotep2(txt )  :
     """url quoter which deals with bytes correctly"""
     btxt = w8enc(txt)
-    quot1 = quote(btxt, safe=b"/")
-    if not PY2:
-        quot2 = quot1.encode("ascii")
-    else:
-        quot2 = quot1
+    quot = quote(btxt, safe=b"/")
+    return w8dec(quot.replace(b" ", b"+"))
 
-    quot3 = quot2.replace(b" ", b"+")
-    return w8dec(quot3)
+
+def _quotep3(txt )  :
+    """url quoter which deals with bytes correctly"""
+    btxt = w8enc(txt)
+    quot = quote(btxt, safe=b"/").encode("utf-8")
+    return w8dec(quot.replace(b" ", b"+"))
+
+
+quotep = _quotep3 if not PY2 else _quotep2
 
 
 def unquotep(txt )  :
@@ -1545,25 +1749,36 @@ def vsplit(vpath )   :
 
 
 def vjoin(rd , fn )  :
-    return rd + "/" + fn if rd else fn
+    if rd and fn:
+        return rd + "/" + fn
+    else:
+        return rd or fn
 
 
-def w8dec(txt )  :
+def _w8dec2(txt )  :
     """decodes filesystem-bytes to wtf8"""
-    if PY2:
-        return surrogateescape.decodefilename(txt)
+    return surrogateescape.decodefilename(txt)
 
+
+def _w8enc2(txt )  :
+    """encodes wtf8 to filesystem-bytes"""
+    return surrogateescape.encodefilename(txt)
+
+
+def _w8dec3(txt )  :
+    """decodes filesystem-bytes to wtf8"""
     return txt.decode(FS_ENCODING, "surrogateescape")
 
 
-def w8enc(txt )  :
+def _w8enc3(txt )  :
     """encodes wtf8 to filesystem-bytes"""
-    if PY2:
-        return surrogateescape.encodefilename(txt)
-
     return txt.encode(FS_ENCODING, "surrogateescape")
 
 
+w8dec = _w8dec3 if not PY2 else _w8dec2
+w8enc = _w8enc3 if not PY2 else _w8enc2
+
+
 def w8b64dec(txt )  :
     """decodes base64(filesystem-bytes) to wtf8"""
     return w8dec(base64.urlsafe_b64decode(txt.encode("ascii")))
@@ -1574,17 +1789,20 @@ def w8b64enc(txt )  :
     return base64.urlsafe_b64encode(w8enc(txt)).decode("ascii")
 
 
-if PY2 and WINDOWS:
-    # moonrunes become \x3f with bytestrings,
-    # losing mojibake support is worth
-    def _not_actually_mbcs(txt):
-        return txt
-
-    fsenc = _not_actually_mbcs
-    fsdec = _not_actually_mbcs
-else:
+if not PY2 or not WINDOWS:
     fsenc = w8enc
     fsdec = w8dec
+else:
+    # moonrunes become \x3f with bytestrings,
+    # losing mojibake support is worth
+    def _not_actually_mbcs_enc(txt )  :
+        return txt
+
+    def _not_actually_mbcs_dec(txt )  :
+        return txt
+
+    fsenc = _not_actually_mbcs_enc
+    fsdec = _not_actually_mbcs_dec
 
 
 def s3enc(mem_cur , rd , fn )   :
@@ -1611,10 +1829,7 @@ def db_ex_chk(log , ex , db_path )  :
     if str(ex) != "database is locked":
         return False
 
-    thr = threading.Thread(target=lsof, args=(log, db_path), name="dbex")
-    thr.daemon = True
-    thr.start()
-
+    Daemon(lsof, "dbex", (log, db_path))
     return True
 
 
@@ -1701,7 +1916,7 @@ def shut_socket(log , sck , timeout  = 3)  :
     finally:
         td = time.time() - t0
         if td >= 1:
-            log("shut({}) in {:.3f} sec".format(fd, td), "1;30")
+            log("shut({}) in {:.3f} sec".format(fd, td), "90")
 
         sck.close()
 
@@ -1786,7 +2001,7 @@ def yieldfile(fn )    :
 
 
 def hashcopy(
-    fin    ,
+    fin   ,
     fout  ,
     slp  = 0,
     max_sz  = 0,
@@ -1858,9 +2073,9 @@ def sendfile_kern(
             n = os.sendfile(out_fd, in_fd, ofs, req)
             stuck = 0
         except OSError as ex:
+            # client stopped reading; do another select
             d = time.time() - stuck
-            log("sendfile stuck for {:.3f} sec: {!r}".format(d, ex))
-            if d < 3600 and ex.errno == 11:  # eagain
+            if d < 3600 and ex.errno == errno.EWOULDBLOCK:
                 continue
 
             n = 0
@@ -1887,6 +2102,7 @@ def statdir(
     if lstat and (PY2 or os.stat not in os.supports_follow_symlinks):
         scandir = False
 
+    src = "statdir"
     try:
         btop = fsenc(top)
         if scandir and hasattr(os, "scandir"):
@@ -2064,7 +2280,7 @@ def killtree(root )  :
             os.kill(pid, signal.SIGTERM)
     else:
         # windows gets minimal effort sorry
-        os.kill(pid, signal.SIGTERM)
+        os.kill(root, signal.SIGTERM)
         return
 
     for n in range(10):
@@ -2086,19 +2302,21 @@ def runcmd(
     kill = ka.pop("kill", "t")  # [t]ree [m]ain [n]one
     capture = ka.pop("capture", 3)  # 0=none 1=stdout 2=stderr 3=both
 
-    sin = ka.pop("sin", None)
+    sin  = ka.pop("sin", None)
     if sin:
         ka["stdin"] = sp.PIPE
 
     cout = sp.PIPE if capture in [1, 3] else None
     cerr = sp.PIPE if capture in [2, 3] else None
+    #bout: bytes
+    #berr: bytes
 
     p = sp.Popen(argv, stdout=cout, stderr=cerr, **ka)
     if not timeout or PY2:
-        stdout, stderr = p.communicate(sin)
+        bout, berr = p.communicate(sin)
     else:
         try:
-            stdout, stderr = p.communicate(sin, timeout=timeout)
+            bout, berr = p.communicate(sin, timeout=timeout)
         except sp.TimeoutExpired:
             if kill == "n":
                 return -18, "", ""  # SIGCONT; leave it be
@@ -2108,15 +2326,15 @@ def runcmd(
                 killtree(p.pid)
 
             try:
-                stdout, stderr = p.communicate(timeout=1)
+                bout, berr = p.communicate(timeout=1)
             except:
-                stdout = b""
-                stderr = b""
+                bout = b""
+                berr = b""
 
-    stdout = stdout.decode("utf-8", "replace") if cout else b""
-    stderr = stderr.decode("utf-8", "replace") if cerr else b""
+    stdout = bout.decode("utf-8", "replace") if cout else ""
+    stderr = berr.decode("utf-8", "replace") if cerr else ""
 
-    rc = p.returncode
+    rc  = p.returncode
     if rc is None:
         rc = -14  # SIGALRM; failed to kill
 
@@ -2301,7 +2519,7 @@ def termsize()   :
     def ioctl_GWINSZ(fd )   :
         try:
             cr = sunpack(b"hh", fcntl.ioctl(fd, termios.TIOCGWINSZ, b"AAAA"))
-            return int(cr[1]), int(cr[0])
+            return cr[::-1]
         except:
             return None
 
@@ -2314,15 +2532,23 @@ def termsize()   :
         except:
             pass
 
-    if cr:
-        return cr
-
     try:
-        return int(env["COLUMNS"]), int(env["LINES"])
+        return cr or (int(env["COLUMNS"]), int(env["LINES"]))
     except:
         return 80, 25
 
 
+def hidedir(dp)  :
+    if ANYWIN:
+        try:
+            k32 = ctypes.WinDLL("kernel32")
+            attrs = k32.GetFileAttributesW(dp)
+            if attrs >= 0:
+                k32.SetFileAttributesW(dp, attrs | 2)
+        except:
+            pass
+
+
 class Pebkac(Exception):
     def __init__(self, code , msg  = None)  :
         super(Pebkac, self).__init__(msg or HTTPCODE[code])
