@@ -27,8 +27,8 @@ window.baguetteBox = (function () {
         isOverlayVisible = false,
         touch = {},  // start-pos
         touchFlag = false,  // busy
-        re_i = /.+\.(gif|jpe?g|png|webp)(\?|$)/i,
-        re_v = /.+\.(webm|mp4)(\?|$)/i,
+        re_i = /.+\.(a?png|avif|bmp|gif|heif|jpe?g|jfif|svg|webp)(\?|$)/i,
+        re_v = /.+\.(webm|mkv|mp4)(\?|$)/i,
         anims = ['slideIn', 'fadeIn', 'none'],
         data = {},  // all galleries
         imagesElements = [],
@@ -246,12 +246,24 @@ window.baguetteBox = (function () {
     }
 
     function keyDownHandler(e) {
-        if (e.ctrlKey || e.altKey || e.metaKey || e.isComposing || modal.busy)
+        if (modal.busy)
+            return;
+
+        if (e.key == '?')
+            return halp();
+
+        if (anymod(e, true))
             return;
 
         var k = e.code + '', v = vid(), pos = -1;
 
-        if (k == "ArrowLeft" || k == "KeyJ")
+        if (k == "BracketLeft")
+            setloop(1);
+        else if (k == "BracketRight")
+            setloop(2);
+        else if (e.shiftKey)
+            return;
+        else if (k == "ArrowLeft" || k == "KeyJ")
             showPreviousImage();
         else if (k == "ArrowRight" || k == "KeyL")
             showNextImage();
@@ -289,10 +301,6 @@ window.baguetteBox = (function () {
             rotn(e.shiftKey ? -1 : 1);
         else if (k == "KeyY")
             dlpic();
-        else if (k == "BracketLeft")
-            setloop(1);
-        else if (k == "BracketRight")
-            setloop(2);
     }
 
     function anim() {
@@ -406,7 +414,7 @@ window.baguetteBox = (function () {
     }
 
     function keyUpHandler(e) {
-        if (e.ctrlKey || e.altKey || e.metaKey || e.isComposing)
+        if (anymod(e))
             return;
 
         var k = e.code + '';
