@@ -672,7 +672,7 @@ function Donut(uc, st) {
             favico.upd();
             wintitle();
             if (document.visibilityState == 'hidden')
-                tenstrobe = setTimeout(enstrobe, 500); //debounce
+                tenstrobe = setTimeout(r.enstrobe, 500); //debounce
         }
     };
 
@@ -709,7 +709,7 @@ function Donut(uc, st) {
         }
     };
 
-    function enstrobe() {
+    r.enstrobe = function () {
         strobes = ['████████████████', '________________', '████████████████'];
         tstrober = setInterval(strobe, 300);
 
@@ -779,7 +779,7 @@ function up2k_init(subtle) {
 
     setTimeout(function () {
         if (window.WebAssembly && !hws.length)
-            fetch('/.cpr/w.hash.js' + CB);
+            fetch(SR + '/.cpr/w.hash.js' + CB);
     }, 1000);
 
     function showmodal(msg) {
@@ -809,7 +809,7 @@ function up2k_init(subtle) {
                 m = L.u_https1 + ' <a href="' + (window.location + '').replace(':', 's:') + '">' + L.u_https2 + '</a> ' + L.u_https3;
 
             showmodal('<h1>loading ' + fn + '</h1>');
-            import_js('/.cpr/deps/' + fn, unmodal);
+            import_js(SR + '/.cpr/deps/' + fn, unmodal);
 
             if (HTTPS) {
                 // chrome<37 firefox<34 edge<12 opera<24 safari<7
@@ -865,9 +865,9 @@ function up2k_init(subtle) {
     bcfg_bind(uc, 'turbo', 'u2turbo', turbolvl > 1, draw_turbo);
     bcfg_bind(uc, 'datechk', 'u2tdate', turbolvl < 3, null);
     bcfg_bind(uc, 'az', 'u2sort', u2sort.indexOf('n') + 1, set_u2sort);
-    bcfg_bind(uc, 'hashw', 'hashw', !!window.WebAssembly && (!subtle || !CHROME || MOBILE), set_hashw);
+    bcfg_bind(uc, 'hashw', 'hashw', !!window.WebAssembly && (!subtle || !CHROME || MOBILE || VCHROME >= 107), set_hashw);
     bcfg_bind(uc, 'upnag', 'upnag', false, set_upnag);
-    bcfg_bind(uc, 'upsfx', 'upsfx', false);
+    bcfg_bind(uc, 'upsfx', 'upsfx', false, set_upsfx);
 
     var st = {
         "files": [],
@@ -904,6 +904,16 @@ function up2k_init(subtle) {
             "u": "",
             "t": ""
         },
+        "etaw": {
+            "h": [['', 0, 0, 0]],
+            "u": [['', 0, 0, 0]],
+            "t": [['', 0, 0, 0]]
+        },
+        "etac": {
+            "h": 0,
+            "u": 0,
+            "t": 0
+        },
         "car": 0,
         "slow_io": null,
         "oserr": false,
@@ -928,7 +938,7 @@ function up2k_init(subtle) {
     r.st = st;
     r.uc = uc;
 
-    if (!window.File || !File.prototype.slice || !window.FileReader || !window.FileList)
+    if (!window.File || !window.FileReader || !window.FileList || !File.prototype || !File.prototype.slice)
         return un2k(L.u_ever);
 
     var flag = false;
@@ -938,15 +948,19 @@ function up2k_init(subtle) {
     function nav() {
         start_actx();
 
+        var uf = function () { ebi('file' + fdom_ctr).click(); },
+            ud = function () { ebi('dir' + fdom_ctr).click(); };
+
         // too buggy on chrome <= 72
         var m = / Chrome\/([0-9]+)\./.exec(navigator.userAgent);
         if (m && parseInt(m[1]) < 73)
-            return ebi('file' + fdom_ctr).click();
+            return uf();
 
-        modal.confirm(L.u_nav_m,
-            function () { ebi('file' + fdom_ctr).click(); },
-            function () { ebi('dir' + fdom_ctr).click(); },
-            null, L.u_nav_b);
+        // phones dont support folder upload
+        if (MOBILE)
+            return uf();
+
+        modal.confirm(L.u_nav_m, uf, ud, null, L.u_nav_b);
     }
     ebi('u2btn').onclick = nav;
 
@@ -1120,7 +1134,7 @@ function up2k_init(subtle) {
                     continue;
 
                 try {
-                    var wi = fobj.webkitGetAsEntry();
+                    var wi = fobj.getAsEntry ? fobj.getAsEntry() : fobj.webkitGetAsEntry();
                     if (wi.isDirectory) {
                         dirs.push(wi);
                         continue;
@@ -1202,7 +1216,7 @@ function up2k_init(subtle) {
                 }
                 else {
                     var name = dn.fullPath;
-                    if (name.indexOf('/') === 0)
+                    if (name.startsWith('/'))
                         name = name.slice(1);
 
                     pf.push(name);
@@ -1225,7 +1239,16 @@ function up2k_init(subtle) {
                 dirs.shift();
                 rd = null;
             }
-            return read_dirs(rd, pf, dirs, good, nil, bad, spins);
+            read_dirs(rd, pf, dirs, good, nil, bad, spins);
+        }, function () {
+            var dn = dirs[0],
+                name = dn.fullPath;
+
+            if (name.startsWith('/'))
+                name = name.slice(1);
+
+            bad.push([dn, name + '/']);
+            read_dirs(null, pf, dirs.slice(1), good, nil, bad, spins);
         });
     }
 
@@ -1299,7 +1322,7 @@ function up2k_init(subtle) {
 
         if (window.WebAssembly && !hws.length) {
             for (var a = 0; a < Math.min(navigator.hardwareConcurrency || 4, 16); a++)
-                hws.push(new Worker('/.cpr/w.hash.js' + CB));
+                hws.push(new Worker(SR + '/.cpr/w.hash.js' + CB));
 
             console.log(hws.length + " hashers");
         }
@@ -1467,10 +1490,20 @@ function up2k_init(subtle) {
             }
         }
         for (var a = 0; a < t.length; a++) {
-            var rem = st.bytes.total - t[a][2],
-                bps = t[a][1] / t[a][3],
-                hid = t[a][0],
+            var hid = t[a][0],
                 eid = hid.slice(-1),
+                etaw = st.etaw[eid];
+
+            if (st.etac[eid] > 100) {  // num chunks
+                st.etac[eid] = 0;
+                etaw.push(jcp(t[a]));
+                if (etaw.length > 5)
+                    etaw.shift();
+            }
+
+            var h = etaw[0],
+                rem = st.bytes.total - t[a][2],
+                bps = (t[a][1] - h[1]) / Math.max(0.1, t[a][3] - h[3]),
                 eta = Math.floor(rem / bps);
 
             if (t[a][1] < 1024 || t[a][3] < 0.1) {
@@ -1495,7 +1528,7 @@ function up2k_init(subtle) {
 
         st.oserr = true;
         var msg = HTTPS ? L.u_emtleak3 : L.u_emtleak2.format((window.location + '').replace(':', 's:'));
-        modal.alert(L.u_emtleak1 + msg + L.u_emtleak4 + (CHROME ? L.u_emtleakc : FIREFOX ? L.u_emtleakf : ''));
+        modal.alert(L.u_emtleak1 + msg + (CHROME ? L.u_emtleakc : FIREFOX ? L.u_emtleakf : ''));
     }
 
     /////
@@ -1800,7 +1833,7 @@ function up2k_init(subtle) {
         while (true) {
             for (var mul = 1; mul <= 2; mul++) {
                 var nchunks = Math.ceil(filesize / chunksize);
-                if (nchunks <= 256 || chunksize >= 32 * 1024 * 1024)
+                if (nchunks <= 256 || (chunksize >= 32 * 1024 * 1024 && nchunks <= 4096))
                     return chunksize;
 
                 chunksize += stepsize;
@@ -1841,6 +1874,7 @@ function up2k_init(subtle) {
                 cdr = Math.min(chunksize + car, t.size);
 
             st.bytes.hashed += cdr - car;
+            st.etac.h++;
 
             function orz(e) {
                 bpend--;
@@ -2088,7 +2122,7 @@ function up2k_init(subtle) {
             try { orz(e); } catch (ex) { vis_exh(ex + '', 'up2k.js', '', '', ex); }
         };
 
-        xhr.open('HEAD', t.purl + uricom_enc(t.name) + '?raw', true);
+        xhr.open('HEAD', t.purl + uricom_enc(t.name), true);
         xhr.send();
     }
 
@@ -2284,8 +2318,8 @@ function up2k_init(subtle) {
                     return;
                 }
 
-                var err_pend = rsp.indexOf('partial upload exists') + 1,
-                    err_dupe = rsp.indexOf('file already exists') + 1;
+                var err_pend = rsp.indexOf('partial upload exists at a different') + 1,
+                    err_dupe = rsp.indexOf('upload rejected, file already exists') + 1;
 
                 if (err_pend || err_dupe) {
                     err = rsp;
@@ -2390,6 +2424,8 @@ function up2k_init(subtle) {
                 st.bytes.finished += cdr - car;
                 st.bytes.uploaded += cdr - car;
                 t.bytes_uploaded += cdr - car;
+                st.etac.u++;
+                st.etac.t++;
             }
             else if (txt.indexOf('already got that') + 1 ||
                 txt.indexOf('already being written') + 1) {
@@ -2497,7 +2533,7 @@ function up2k_init(subtle) {
     tt.att(QS('#u2conf'));
 
     function bumpthread2(e) {
-        if (e.ctrlKey || e.altKey || e.metaKey || e.isComposing)
+        if (anymod(e))
             return;
 
         if (e.code == 'ArrowUp')
@@ -2571,6 +2607,7 @@ function up2k_init(subtle) {
         el.innerHTML = '<div>' + L.u_life_cfg + '</div><div>' + L.u_life_est + '</div><div id="undor"></div>';
         set_life(Math.min(lifetime, icfg_get('lifetime', lifetime)));
         ebi('lifem').oninput = ebi('lifeh').oninput = mod_life;
+        ebi('lifem').onkeydown = ebi('lifeh').onkeydown = kd_life;
         tt.att(ebi('u2life'));
     }
     draw_life();
@@ -2596,12 +2633,23 @@ function up2k_init(subtle) {
         set_life(v);
     }
 
+    function kd_life(e) {
+        var el = e.target,
+            d = e.code == 'ArrowUp' ? 1 : e.code == 'ArrowDown' ? -1 : 0;
+
+        if (anymod(e) || !d)
+            return;
+
+        el.value = parseInt(el.value) + d;
+        mod_life(e);
+    }
+
     function set_life(v) {
         //ebi('lifes').value = v;
         ebi('lifem').value = parseInt(v / 60);
         ebi('lifeh').value = parseInt(v / 3600);
 
-        var undo = have_unpost - (v || lifetime);
+        var undo = have_unpost - (v ? lifetime - v : 0);
         ebi('undor').innerHTML = undo <= 0 ?
             L.u_unp_ng : L.u_unp_ok.format(lhumantime(undo));
 
@@ -2724,6 +2772,21 @@ function up2k_init(subtle) {
 
         if (en && Notification.permission == 'default')
             Notification.requestPermission().then(chknag, chknag);
+
+        set_upsfx(en);
+    }
+
+    function set_upsfx(en) {
+        if (!en)
+            return;
+
+        toast.inf(10, 'OK -- <a href="#" id="nagtest">test it!</a>')
+
+        ebi('nagtest').onclick = function () {
+            start_actx();
+            uc.nagtxt = ':^)';
+            setTimeout(donut.enstrobe, 200);
+        };
     }
 
     if (uc.upnag && (!window.Notification || Notification.permission != 'granted'))
